@@ -22,7 +22,8 @@ import distro_info
 from functools import wraps
 import gi
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gio, Gtk
+gi.require_version("Handy", "1")
+from gi.repository import Gio, Gtk, GLib, Handy
 import json
 import os
 import subprocess
@@ -78,7 +79,7 @@ def get_ua_status():
     # status.json will exist on any attached system. It will also be created
     # by the systemd timer ua-timer which will update UA_STATUS_JSON every 12
     # hours to reflect current status of UA subscription services.
-    # Invoking `ua status` with subp will result in a network call to
+    # Invoking `pro status` with subp will result in a network call to
     # contracts.canonical.com which could raise Timeouts on network limited
     # machines. So, prefer the status.json file when possible.
     status_json = ""
@@ -89,21 +90,21 @@ def get_ua_status():
         try:
             # Success writes UA_STATUS_JSON
             result = subprocess.run(
-                ['ua', 'status', '--format=json'], stdout=subprocess.PIPE
+                ['pro', 'status', '--format=json'], stdout=subprocess.PIPE
             )
         except Exception as e:
-            print("Failed to run `ua status`:\n%s" % e)
+            print("Failed to run `pro status`:\n%s" % e)
             return {}
         if result.returncode != 0:
             print(
-                "Ubuntu Advantage client returned code %d" % result.returncode
+                "Ubuntu Pro client returned code %d" % result.returncode
             )
             return {}
         status_json = result.stdout
     if not status_json:
         print(
-            "Warning: no Ubuntu Advantage status found."
-            " Is ubuntu-advantage-tools installed?"
+            "Warning: no Ubuntu Pro Client status found."
+            " Is ubuntu-pro-client installed?"
         )
         return {}
     try:
@@ -113,7 +114,7 @@ def get_ua_status():
         return {}
     if status.get("_schema_version", "0.1") != "0.1":
         print(
-            "UA status schema version change: %s" % status["_schema_version"]
+            "Pro status schema version change: %s" % status["_schema_version"]
         )
     return status
 
@@ -178,3 +179,9 @@ def retry(exceptions, tries=10, delay=0.1, backoff=2):
         return f_retry  # true decorator
 
     return deco_retry
+
+def is_dark_theme(widget):
+    env_gtk_theme = GLib.getenv("GTK_THEME")
+    if env_gtk_theme != None:
+        return GLib.str_has_suffix(env_gtk_theme, "dark")
+    return Handy.StyleManager.get_default().get_dark()
