@@ -22,19 +22,18 @@ import subprocess
 import tempfile
 
 from aptsources.distro import get_distro
-
-from softwareproperties.extendedsourceslist import (SourceEntry,
-                                                    SourcesList,
-                                                    CollapsedSourcesList)
+from aptsources.sourceslist import (SourceEntry, SourcesList, Deb822SourceEntry)
 
 from contextlib import suppress
 
-from copy import copy
-
 from gettext import gettext as _
 
 from urllib.parse import urlparse
 
+from softwareproperties.sourceutils import (get_source_entry_from_list,
+                                            copy_source_entry,
+                                            deb822_source_entry_contains)
+
 
 apt_pkg.init()
 
@@ -53,7 +52,8 @@ class ShortcutHandler(object):
     should be modified.  The only exception to that rule is adding or removing
     sourceslist lines or components of existing source entries.
     '''
-    def __init__(self, shortcut, components=None, enable_source=False, codename=None, pocket=None, dry_run=False, **kwargs):
+    def __init__(self, shortcut, components=None, enable_source=False,
+                 codename=None, pocket=None, dry_run=False, deb822=False, **kwargs):
         self.shortcut = shortcut
         self.components = components or []
         self.enable_source = enable_source
@@ -61,6 +61,7 @@ class ShortcutHandler(object):
         self.codename = codename or self.distro.codename
         self.pocket = pocket
         self.dry_run = dry_run
+        self.deb822 = deb822
 
         # Subclasses should not directly reference _source_entry,
         # use _set_source_entry() and SourceEntry()
@@ -174,18 +175,22 @@ class ShortcutHandler(object):
         '''
         if not self._source_entry:
             raise NotImplementedError('Implementation class did not set self._source_entry')
-        e = copy(self._source_entry)
+        e = copy_source_entry(self._source_entry)
         if not pkgtype:
             return e
         if pkgtype == self.binary_type:
             e.set_enabled(True)
-            e.type = self.binary_type
         elif pkgtype == self.source_type:
             e.set_enabled(self.enable_source)
-            e.type = self.source_type
         else:
             raise ValueError('Invalid pkgtype: %s' % pkgtype)
-        return SourceEntry(str(e), file=e.file)
+
+        if isinstance(e, Deb822SourceEntry):
+            e.types = [pkgtype]
+        else:
+            e.type = pkgtype
+
+        return copy_source_entry(e)
 
     @property
     def username(self):
@@ -233,6 +238,65 @@ class ShortcutHandler(object):
         self.remove_source()
         self.remove_login()
 
+    def _add_deb822_source(self):
+        sourceslist = SourcesList(deb822=True)
+
+        newentry = None
+        binentry = self.SourceEntry(self.binary_type)
+        for s in sourceslist:
+            if not isinstance(s, Deb822SourceEntry):
+                continue
+
+            if deb822_source_entry_contains(s, binentry):
+                newentry = s
+                break
+
+        if newentry is not None:
+            print(_("Found existing %s entry in %s") % (self.binary_type,
+                                                        newentry.file))
+        else:
+            newentry = sourceslist.add('', '', '', [],
+                                       file=self.sourceparts_file)
+            newentry.types = binentry.types
+            newentry.uris = binentry.uris
+            newentry.suites = binentry.suites
+            newentry.comps = binentry.comps
+
+            if self.trustedparts_content:
+                lines = self.trustedparts_content.splitlines()
+                lines = [' ' + (l if l.strip() else '.') for l in lines]
+                newentry.section['Signed-By'] = '\n' + '\n'.join(lines)
+
+        binentry = newentry
+
+        if self.enable_source and self.source_type not in set(binentry.types):
+            newentry = None
+            srcentry = self.SourceEntry(self.source_type)
+
+            for s in sourceslist:
+                if not isinstance(s, Deb822SourceEntry):
+                    continue
+
+                if deb822_source_entry_contains(s, srcentry):
+                    newentry = s
+                    break
+
+            if newentry is not None:
+                print(_("Found existing %s entry in %s") % (self.source_type,
+                                                            newentry.file))
+            else:
+                binentry.types = [self.binary_type, self.source_type]
+
+        if not self.dry_run:
+            if not os.path.exists(binentry.file):
+                # Create the dir if needed
+                if (binentry.file.startswith(self.sourceparts_path)
+                    and not os.path.exists(self.sourceparts_path)):
+                    os.mkdir(self.sourceparts_path, 0o755)
+                with open(binentry.file, 'w'):
+                    os.chmod(binentry.file, self.sourceparts_mode)
+            sourceslist.save()
+
     def add_source(self):
         '''Add the apt SourceEntries.
 
@@ -248,18 +312,28 @@ class ShortcutHandler(object):
         SourcesList, the existing entries are updated instead of placing
         the entries in the sourceparts_file.
         '''
+        if self.deb822:
+            self._add_deb822_source()
+            return
+
         binentry = self.SourceEntry(self.binary_type)
         srcentry = self.SourceEntry(self.source_type)
         mode = self.sourceparts_mode
 
         sourceslist = SourcesList()
-        collapsedlist = CollapsedSourcesList(sourceslist)
 
-        newentry = collapsedlist.get_entry(binentry)
-        if newentry:
+        newentry = get_source_entry_from_list(sourceslist, binentry)
+        if newentry is not None:
             print(_("Found existing %s entry in %s") % (newentry.type, newentry.file))
         else:
-            newentry = collapsedlist.add_entry(binentry)
+            newentry = sourceslist.add(
+                uri=binentry.uri,
+                dist=binentry.dist,
+                file=binentry.file,
+                type=binentry.type,
+                orig_comps=binentry.comps
+            )
+            newentry.disabled = binentry.disabled
 
         if binentry.file != newentry.file:
             # existing binentry, but not in file we were expecting, just update it
@@ -278,11 +352,18 @@ class ShortcutHandler(object):
         # Unless it already exists somewhere, add the srcentry right after the binentry
         srcentry.file = binentry.file
 
-        newentry = collapsedlist.get_entry(srcentry)
-        if newentry:
+        newentry = get_source_entry_from_list(sourceslist, srcentry)
+        if newentry is not None:
             print(_("Found existing %s entry in %s") % (newentry.type, newentry.file))
         else:
-            newentry = collapsedlist.add_entry(srcentry, after=binentry)
+            newentry = sourceslist.add(
+                uri=srcentry.uri,
+                dist=srcentry.dist,
+                file=srcentry.file,
+                type=srcentry.type,
+                orig_comps=srcentry.comps
+            )
+            newentry.disabled = srcentry.disabled
 
         if srcentry.file != newentry.file:
             # existing srcentry, but not in file we were expecting, just update it
@@ -306,6 +387,28 @@ class ShortcutHandler(object):
                         os.chmod(entryfile, mode)
             sourceslist.save()
 
+    def _remove_deb822_source(self):
+        sourceslist = SourcesList(deb822=True)
+        binentry = self.SourceEntry(self.binary_type)
+
+        for s in sourceslist:
+            if not isinstance(s, Deb822SourceEntry):
+                continue
+
+            if deb822_source_entry_contains(s, binentry):
+                print(_("Removing entry from %s") % (s.file))
+                binentry = s
+                sourceslist.remove(binentry)
+                break
+
+        if not self.dry_run:
+            sourceslist.save()
+            if not [s for s in sourceslist if s.file == binentry.file]:
+                try:
+                    os.remove(binentry.file)
+                except (OSError, IOError):
+                    pass
+
     def remove_source(self):
         '''Remove the apt SourceEntries.
 
@@ -321,21 +424,23 @@ class ShortcutHandler(object):
         empty or contains only invalid and/or disabled SourceEntries, this
         may remove the sourceparts_file.
         '''
+        if self.deb822:
+            self._remove_deb822_source()
+            return
+
         sourceslist = SourcesList()
-        collapsedlist = CollapsedSourcesList(sourceslist)
+        orig_files = set([e.file for e in sourceslist])
 
         binentry = self.SourceEntry(self.binary_type)
         srcentry = self.SourceEntry(self.source_type)
 
-        # Disable the entries
-        binentry.set_enabled(True)
-        if collapsedlist.has_entry(binentry):
-            print(_("Disabling %s entry in %s") % (binentry.type, binentry.file))
-            collapsedlist.add_entry(binentry._replace(disabled=True))
-        srcentry.set_enabled(True)
-        if collapsedlist.has_entry(srcentry):
-            print(_("Disabling %s entry in %s") % (srcentry.type, srcentry.file))
-            collapsedlist.add_entry(srcentry._replace(disabled=True))
+        binentry.disabled = False
+        srcentry.disabled = False
+
+        for s in sourceslist:
+            if get_source_entry_from_list([binentry,srcentry], s) is not None:
+                print(_("Disabling %s entry in %s") % (s.type, s.file))
+                s.disabled = True
 
         file_entries = [s for s in sourceslist if s.file == self.sourceparts_file]
         if not [e for e in file_entries if not e.invalid and not e.disabled]:
@@ -346,7 +451,12 @@ class ShortcutHandler(object):
                 sourceslist.remove(e)
 
         if not self.dry_run:
-            sourceslist.save(remove=True)
+            sourceslist.save()
+            for file in orig_files - set([e.file for e in sourceslist]):
+                try:
+                    os.remove(file)
+                except (OSError, IOError):
+                    pass
 
     @property
     def sourceparts_path(self):
@@ -358,10 +468,12 @@ class ShortcutHandler(object):
         '''Get the sources.list.d filename, without the leading path.
 
         By default, this combines the filebase with the codename, and uses a
-        extension of 'list'.  This is different than the trustedparts or
+        extension of 'list' if deb822 is not enabled, and 'sources' if deb822
+        is enabled.  This is different than the trustedparts or
         netrcparts filenames, which use only the filebase plus extension.
         '''
-        return self._filebase_to_filename('list', suffix=self.codename)
+        ext = 'sources' if self.deb822 else 'list'
+        return self._filebase_to_filename(ext, suffix=self.codename)
 
     @property
     def sourceparts_file(self):
@@ -395,6 +507,10 @@ class ShortcutHandler(object):
         If the file does not yet exist, and self.trustedparts_mode is set,
         the file will be created with that mode.
         '''
+        if self.deb822:
+            # The key is embedded into the file when the source is added.
+            return
+
         if not all((self.trustedparts_file, self.trustedparts_content)):
             return
 
@@ -610,9 +726,13 @@ class ShortcutHandler(object):
 
         The self.components, if any, will be added to the line's component(s).
         '''
-        e = SourceEntry(line)
+        if self.deb822:
+            e = Deb822SourceEntry(line, file=self.sourceparts_file)
+        else:
+            e = SourceEntry(line, file=self.sourceparts_file)
+
         e.comps = list(set(e.comps) | set(self.components))
-        self._source_entry = SourceEntry(str(e), file=self.sourceparts_file)
+        self._source_entry = copy_source_entry(e)
 
     def _encode_filebase(self, suffix=None):
         base = self._filebase
